@@ -2,7 +2,6 @@
  * http.c : HTTP/HTTPS Remote control interface
  *****************************************************************************
  * Copyright (C) 2001-2006 the VideoLAN team
- * $Id: http.c 15102 2006-04-05 09:58:20Z xtophe $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -22,8 +21,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "http.h"
+#include <vlc_plugin.h>
+
+#include <assert.h>
 
 /*****************************************************************************
  * Module descriptor
@@ -39,13 +44,14 @@ static void Close( vlc_object_t * );
     "machine, enter 127.0.0.1" )
 #define SRC_TEXT N_( "Source directory" )
 #define SRC_LONGTEXT N_( "Source directory" )
-#define CHARSET_TEXT N_( "Charset" )
-#define CHARSET_LONGTEXT N_( \
-        "Charset declared in Content-Type header (default UTF-8)." )
 #define HANDLERS_TEXT N_( "Handlers" )
 #define HANDLERS_LONGTEXT N_( \
         "List of handler extensions and executable paths (for instance: " \
         "php=/usr/bin/php,pl=/usr/bin/perl)." )
+#define ART_TEXT N_( "Export album art as /art." )
+#define ART_LONGTEXT N_( \
+        "Allow exporting album art for current playlist items at the " \
+        "/art and /art?id=<id> URLs." )
 #define CERT_TEXT N_( "Certificate file" )
 #define CERT_LONGTEXT N_( "HTTP interface x509 PEM certificate file " \
                           "(enables SSL)." )
@@ -58,21 +64,22 @@ static void Close( vlc_object_t * );
 #define CRL_LONGTEXT N_( "HTTP interace Certificates Revocation List file." )
 
 vlc_module_begin();
-    set_shortname( _("HTTP"));
-    set_description( _("HTTP remote control interface") );
+    set_shortname( N_("HTTP"));
+    set_description( N_("HTTP remote control interface") );
     set_category( CAT_INTERFACE );
     set_subcategory( SUBCAT_INTERFACE_MAIN );
-        add_string ( "http-host", NULL, NULL, HOST_TEXT, HOST_LONGTEXT, VLC_TRUE );
-        add_string ( "http-src",  NULL, NULL, SRC_TEXT,  SRC_LONGTEXT,  VLC_TRUE );
-        add_string ( "http-charset", "UTF-8", NULL, CHARSET_TEXT, CHARSET_LONGTEXT, VLC_TRUE );
+        add_string ( "http-host", NULL, NULL, HOST_TEXT, HOST_LONGTEXT, true );
+        add_string ( "http-src",  NULL, NULL, SRC_TEXT,  SRC_LONGTEXT,  true );
+        add_obsolete_string ( "http-charset" );
 #if defined( HAVE_FORK ) || defined( WIN32 )
-        add_string ( "http-handlers", NULL, NULL, HANDLERS_TEXT, HANDLERS_LONGTEXT, VLC_TRUE );
+        add_string ( "http-handlers", NULL, NULL, HANDLERS_TEXT, HANDLERS_LONGTEXT, true );
 #endif
+        add_bool   ( "http-album-art", false, NULL, ART_TEXT, ART_LONGTEXT, true );
         set_section( N_("HTTP SSL" ), 0 );
-        add_string ( "http-intf-cert", NULL, NULL, CERT_TEXT, CERT_LONGTEXT, VLC_TRUE );
-        add_string ( "http-intf-key",  NULL, NULL, KEY_TEXT,  KEY_LONGTEXT,  VLC_TRUE );
-        add_string ( "http-intf-ca",   NULL, NULL, CA_TEXT,   CA_LONGTEXT,   VLC_TRUE );
-        add_string ( "http-intf-crl",  NULL, NULL, CRL_TEXT,  CRL_LONGTEXT,  VLC_TRUE );
+        add_string ( "http-intf-cert", NULL, NULL, CERT_TEXT, CERT_LONGTEXT, true );
+        add_string ( "http-intf-key",  NULL, NULL, KEY_TEXT,  KEY_LONGTEXT,  true );
+        add_string ( "http-intf-ca",   NULL, NULL, CA_TEXT,   CA_LONGTEXT,   true );
+        add_string ( "http-intf-crl",  NULL, NULL, CRL_TEXT,  CRL_LONGTEXT,  true );
     set_capability( "interface", 0 );
     set_callbacks( Open, Close );
 vlc_module_end();
@@ -81,36 +88,12 @@ vlc_module_end();
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static void Run          ( intf_thread_t *p_intf );
-
-/*****************************************************************************
- * Local functions
- *****************************************************************************/
-#if !defined(__APPLE__) && !defined(SYS_BEOS) && !defined(WIN32)
-static int DirectoryCheck( const char *psz_dir )
-{
-    DIR           *p_dir;
-
-#ifdef HAVE_SYS_STAT_H
-    struct stat   stat_info;
-
-    if( ( utf8_stat( psz_dir, &stat_info ) == -1 )
-      || !S_ISDIR( stat_info.st_mode ) )
-    {
-        return VLC_EGENERIC;
-    }
-#endif
-
-    if( ( p_dir = utf8_opendir( psz_dir ) ) == NULL )
-    {
-        return VLC_EGENERIC;
-    }
-    closedir( p_dir );
-
-    return VLC_SUCCESS;
-}
-#endif
-
+int  ArtCallback( httpd_handler_sys_t *p_args,
+                          httpd_handler_t *p_handler, char *_p_url,
+                          uint8_t *_p_request, int i_type,
+                          uint8_t *_p_in, int i_in,
+                          char *psz_remote_addr, char *psz_remote_host,
+                          uint8_t **pp_data, int *pi_data );
 
 /*****************************************************************************
  * Activate: initialize and create stuff
@@ -120,16 +103,15 @@ static int Open( vlc_object_t *p_this )
     intf_thread_t *p_intf = (intf_thread_t*)p_this;
     intf_sys_t    *p_sys;
     char          *psz_address;
-    const char    *psz_cert = NULL, *psz_key = NULL, *psz_ca = NULL,
+    char          *psz_cert = NULL, *psz_key = NULL, *psz_ca = NULL,
                   *psz_crl = NULL;
     int           i_port       = 0;
-    char          *psz_src;
-    char          psz_tmp[10];
+    char          *psz_src = NULL;
 
-    psz_address = config_GetPsz( p_intf, "http-host" );
+    psz_address = var_CreateGetNonEmptyString( p_intf, "http-host" );
     if( psz_address != NULL )
     {
-        char *psz_parser = strchr( psz_address, ':' );
+        char *psz_parser = strrchr( psz_address, ':' );
         if( psz_parser )
         {
             *psz_parser++ = '\0';
@@ -142,58 +124,16 @@ static int Open( vlc_object_t *p_this )
     p_intf->p_sys = p_sys = malloc( sizeof( intf_sys_t ) );
     if( !p_intf->p_sys )
     {
+        free( psz_address );
         return( VLC_ENOMEM );
     }
-    p_sys->p_playlist = NULL;
+
+    p_sys->p_playlist = pl_Yield( p_this );
     p_sys->p_input    = NULL;
     p_sys->p_vlm      = NULL;
     p_sys->psz_address = psz_address;
     p_sys->i_port     = i_port;
-
-    /* determine Content-Type value for HTML pages */
-    psz_src = config_GetPsz( p_intf, "http-charset" );
-    if( psz_src == NULL || !*psz_src )
-    {
-        if( psz_src != NULL ) free( psz_src );
-        psz_src = strdup("UTF-8");
-    }
-
-    p_sys->psz_html_type = malloc( 20 + strlen( psz_src ) );
-    if( p_sys->psz_html_type == NULL )
-    {
-        free( p_sys->psz_address );
-        free( p_sys );
-        free( psz_src );
-        return VLC_ENOMEM ;
-    }
-    sprintf( p_sys->psz_html_type, "text/html; charset=%s", psz_src );
-    msg_Dbg( p_intf, "using charset=%s", psz_src );
-
-    if( strcmp( psz_src, "UTF-8" ) )
-    {
-        char psz_encoding[strlen( psz_src ) + sizeof( "//translit")];
-        sprintf( psz_encoding, "%s//translit", psz_src);
-
-        p_sys->iconv_from_utf8 = vlc_iconv_open( psz_encoding, "UTF-8" );
-        if( p_sys->iconv_from_utf8 == (vlc_iconv_t)-1 )
-            msg_Warn( p_intf, "unable to perform charset conversion to %s",
-                      psz_encoding );
-        else
-        {
-            p_sys->iconv_to_utf8 = vlc_iconv_open( "UTF-8", psz_src );
-            if( p_sys->iconv_to_utf8 == (vlc_iconv_t)-1 )
-                msg_Warn( p_intf,
-                          "unable to perform charset conversion from %s",
-                          psz_src );
-        }
-    }
-    else
-    {
-        p_sys->iconv_from_utf8 = p_sys->iconv_to_utf8 = (vlc_iconv_t)-1;
-    }
-
-    p_sys->psz_charset = psz_src;
-    psz_src = NULL;
+    p_sys->p_art_handler = NULL;
 
     /* determine file handler associations */
     p_sys->i_handlers = 0;
@@ -218,14 +158,14 @@ static int Open( vlc_object_t *p_this )
 
             p_handler = malloc( sizeof( http_association_t ) );
             p_handler->psz_ext = strdup( psz_ext );
-            psz_options = E_(FirstWord)( psz_program, psz_program );
+            psz_options = FirstWord( psz_program, psz_program );
             p_handler->i_argc = 0;
             p_handler->ppsz_argv = NULL;
             TAB_APPEND( p_handler->i_argc, p_handler->ppsz_argv,
                         strdup( psz_program ) );
             while( psz_options != NULL && *psz_options )
             {
-                char *psz_next = E_(FirstWord)( psz_options, psz_options );
+                char *psz_next = FirstWord( psz_options, psz_options );
                 TAB_APPEND( p_handler->i_argc, p_handler->ppsz_argv,
                             strdup( psz_options ) );
                 psz_options = psz_next;
@@ -235,8 +175,7 @@ static int Open( vlc_object_t *p_this )
             TAB_APPEND( p_sys->i_handlers, p_sys->pp_handlers, p_handler );
         }
     }
-    if( psz_src != NULL )
-        free( psz_src );
+    free( psz_src );
 #endif
 
     /* determine SSL configuration */
@@ -245,9 +184,9 @@ static int Open( vlc_object_t *p_this )
     {
         msg_Dbg( p_intf, "enabling TLS for HTTP interface (cert file: %s)",
                  psz_cert );
-        psz_key = config_GetPsz( p_intf, "http-intf-key" );
-        psz_ca = config_GetPsz( p_intf, "http-intf-ca" );
-        psz_crl = config_GetPsz( p_intf, "http-intf-crl" );
+        psz_key = var_GetNonEmptyString( p_intf, "http-intf-key" );
+        psz_ca = var_GetNonEmptyString( p_intf, "http-intf-ca" );
+        psz_crl = var_GetNonEmptyString( p_intf, "http-intf-crl" );
 
         if( i_port <= 0 )
             i_port = 8443;
@@ -258,66 +197,44 @@ static int Open( vlc_object_t *p_this )
             i_port= 8080;
     }
 
-    /* Ugly hack to allow to run several HTTP servers on different ports. */
-    sprintf( psz_tmp, ":%d", i_port + 1 );
-    config_PutPsz( p_intf, "http-host", psz_tmp );
-
     msg_Dbg( p_intf, "base %s:%d", psz_address, i_port );
 
     p_sys->p_httpd_host = httpd_TLSHostNew( VLC_OBJECT(p_intf), psz_address,
                                             i_port, psz_cert, psz_key, psz_ca,
                                             psz_crl );
+    free( psz_cert );
+    free( psz_key );
+    free( psz_ca );
+    free( psz_crl );
+
     if( p_sys->p_httpd_host == NULL )
     {
         msg_Err( p_intf, "cannot listen on %s:%d", psz_address, i_port );
-        free( p_sys->psz_html_type );
+        pl_Release( p_this );
         free( p_sys->psz_address );
         free( p_sys );
         return VLC_EGENERIC;
     }
+    else
+    {
+        char psz_tmp[NI_MAXHOST + 6];
+
+        /* Ugly hack to run several HTTP servers on different ports */
+        snprintf( psz_tmp, sizeof (psz_tmp), "%s:%d", psz_address, i_port + 1 );
+        var_Create(p_intf->p_libvlc, "http-host", VLC_VAR_STRING );
+        var_SetString( p_intf->p_libvlc, "http-host", psz_tmp );
+    }
 
     p_sys->i_files  = 0;
     p_sys->pp_files = NULL;
 
-#if defined(__APPLE__) || defined(SYS_BEOS) || defined(WIN32)
-    if ( ( psz_src = config_GetPsz( p_intf, "http-src" )) == NULL )
-    {
-        char * psz_vlcpath = p_intf->p_libvlc->psz_vlcpath;
-        psz_src = malloc( strlen(psz_vlcpath) + strlen("/share/http" ) + 1 );
-        if( !psz_src ) return VLC_ENOMEM;
-#if defined(WIN32)
-        sprintf( psz_src, "%s/http", psz_vlcpath );
-#else
-        sprintf( psz_src, "%s/share/http", psz_vlcpath );
-#endif
-    }
-#else
     psz_src = config_GetPsz( p_intf, "http-src" );
-
     if( ( psz_src == NULL ) || ( *psz_src == '\0' ) )
     {
-        static char const* ppsz_paths[] = {
-            "share/http",
-            "../share/http",
-            DATA_PATH"/http",
-            NULL
-        };
-        unsigned i;
-
-        if( psz_src != NULL )
-        {
-            free( psz_src );
+        const char *data_path = config_GetDataDir ();
+        if( asprintf( &psz_src, "%s" DIR_SEP "http", data_path ) == -1 )
             psz_src = NULL;
-        }
-
-        for( i = 0; ppsz_paths[i] != NULL; i++ )
-            if( !DirectoryCheck( ppsz_paths[i] ) )
-            {
-                psz_src = strdup( ppsz_paths[i] );
-                break;
-            }
     }
-#endif
 
     if( !psz_src || *psz_src == '\0' )
     {
@@ -332,33 +249,43 @@ static int Open( vlc_object_t *p_this )
         psz_src[strlen( psz_src ) - 1] = '\0';
     }
 
-    E_(ParseDirectory)( p_intf, psz_src, psz_src );
-
-
+    ParseDirectory( p_intf, psz_src, psz_src );
     if( p_sys->i_files <= 0 )
     {
         msg_Err( p_intf, "cannot find any file in directory %s", psz_src );
         goto failed;
     }
-    p_intf->pf_run = Run;
+
     free( psz_src );
 
+    if( config_GetInt( p_intf, "http-album-art" ) )
+    {
+        /* FIXME: we're leaking h */
+        httpd_handler_sys_t *h = malloc( sizeof( httpd_handler_sys_t ) );
+        if( !h )
+        {
+            msg_Err( p_intf, "not enough memory to allocate album art handler" );
+            goto failed;
+        }
+        h->file.p_intf = p_intf;
+        h->file.file = NULL;
+        h->file.name = NULL;
+        /* TODO: use ACL and login/password stuff here too */
+        h->p_handler = httpd_HandlerNew( p_sys->p_httpd_host,
+                                         "/art", NULL, NULL, NULL,
+                                         ArtCallback, h );
+        p_sys->p_art_handler = h->p_handler;
+    }
+
     return VLC_SUCCESS;
 
 failed:
-    if( psz_src ) free( psz_src );
-    if( p_sys->pp_files )
-    {
-        free( p_sys->pp_files );
-    }
+    free( psz_src );
+    free( p_sys->pp_files );
     httpd_HostDelete( p_sys->p_httpd_host );
     free( p_sys->psz_address );
-    free( p_sys->psz_html_type );
-    if( p_sys->iconv_from_utf8 != (vlc_iconv_t)-1 )
-        vlc_iconv_close( p_sys->iconv_from_utf8 );
-    if( p_sys->iconv_to_utf8 != (vlc_iconv_t)-1 )
-        vlc_iconv_close( p_sys->iconv_to_utf8 );
     free( p_sys );
+    pl_Release( p_this );
     return VLC_EGENERIC;
 }
 
@@ -369,13 +296,12 @@ static void Close ( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
     intf_sys_t    *p_sys = p_intf->p_sys;
-
     int i;
 
+#ifdef ENABLE_VLM
     if( p_sys->p_vlm )
-    {
         vlm_Delete( p_sys->p_vlm );
-    }
+#endif
     for( i = 0; i < p_sys->i_files; i++ )
     {
         if( p_sys->pp_files[i]->b_handler )
@@ -391,10 +317,7 @@ static void Close ( vlc_object_t *p_this )
         free( p_sys->pp_files[i]->name );
         free( p_sys->pp_files[i] );
     }
-    if( p_sys->pp_files )
-    {
-        free( p_sys->pp_files );
-    }
+    free( p_sys->pp_files );
     for( i = 0; i < p_sys->i_handlers; i++ )
     {
         http_association_t *p_handler = p_sys->pp_handlers[i];
@@ -408,61 +331,12 @@ static void Close ( vlc_object_t *p_this )
     }
     if( p_sys->i_handlers )
         free( p_sys->pp_handlers );
+    if( p_sys->p_art_handler )
+        httpd_HandlerDelete( p_sys->p_art_handler );
     httpd_HostDelete( p_sys->p_httpd_host );
     free( p_sys->psz_address );
-    free( p_sys->psz_html_type );
-
-    if( p_sys->iconv_from_utf8 != (vlc_iconv_t)-1 )
-        vlc_iconv_close( p_sys->iconv_from_utf8 );
-    if( p_sys->iconv_to_utf8 != (vlc_iconv_t)-1 )
-        vlc_iconv_close( p_sys->iconv_to_utf8 );
     free( p_sys );
-}
-
-/*****************************************************************************
- * Run: http interface thread
- *****************************************************************************/
-static void Run( intf_thread_t *p_intf )
-{
-    intf_sys_t     *p_sys = p_intf->p_sys;
-
-    while( !p_intf->b_die )
-    {
-        /* get the playlist */
-        if( p_sys->p_playlist == NULL )
-        {
-            p_sys->p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
-        }
-
-        /* Manage the input part */
-        if( p_sys->p_input == NULL )
-        {
-            if( p_sys->p_playlist )
-            {
-                p_sys->p_input = p_sys->p_playlist->p_input;
-            }
-        }
-        else if( p_sys->p_input->b_dead || p_sys->p_input->b_die )
-        {
-            p_sys->p_input = NULL;
-        }
-
-
-        /* Wait a bit */
-        msleep( INTF_IDLE_SLEEP );
-    }
-
-    if( p_sys->p_input )
-    {
-        vlc_object_release( p_sys->p_input );
-        p_sys->p_input = NULL;
-    }
-
-    if( p_sys->p_playlist )
-    {
-        vlc_object_release( p_sys->p_playlist );
-        p_sys->p_playlist = NULL;
-    }
+    pl_Release( p_this );
 }
 
 /****************************************************************************
@@ -478,13 +352,13 @@ static void Callback404( httpd_file_sys_t *p_args, char **pp_data,
     {
         return;
     }
-    p += sprintf( p, "<html>\n" );
+    p += sprintf( p, "Content-Type: text/html\n" );
+    p += sprintf( p, "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n" );
     p += sprintf( p, "<head>\n" );
     p += sprintf( p, "<title>Error loading %s</title>\n", p_args->file );
     p += sprintf( p, "</head>\n" );
     p += sprintf( p, "<body>\n" );
     p += sprintf( p, "<h1><center>Error loading %s for %s</center></h1>\n", p_args->file, p_args->name );
-    p += sprintf( p, "<hr />\n" );
     p += sprintf( p, "<a href=\"http://www.videolan.org/\">VideoLAN</a>\n" );
     p += sprintf( p, "</body>\n" );
     p += sprintf( p, "</html>\n" );
@@ -496,6 +370,7 @@ static void ParseExecute( httpd_file_sys_t *p_args, char *p_buffer,
                           int i_buffer, char *p_request,
                           char **pp_data, int *pi_data )
 {
+    intf_sys_t *p_sys = p_args->p_intf->p_sys;
     int i_request = p_request != NULL ? strlen( p_request ) : 0;
     char *dst;
     vlc_value_t val;
@@ -504,31 +379,42 @@ static void ParseExecute( httpd_file_sys_t *p_args, char *p_buffer,
     char length[12]; /* in seconds */
     audio_volume_t i_volume;
     char volume[5];
-    char state[8];
+    const char *state;
     char stats[20];
 
-#define p_sys p_args->p_intf->p_sys
+    assert( p_sys->p_input == NULL );
+    /* FIXME: proper locking anyone? */
+    p_sys->p_input = p_sys->p_playlist->p_input;
     if( p_sys->p_input )
     {
+        vlc_object_yield( p_sys->p_input );
         var_Get( p_sys->p_input, "position", &val);
         sprintf( position, "%d" , (int)((val.f_float) * 100.0));
         var_Get( p_sys->p_input, "time", &val);
-        sprintf( time, "%d" , (int)(val.i_time / 1000000) );
+        sprintf( time, "%"PRIi64, (int64_t)val.i_time / INT64_C(1000000) );
         var_Get( p_sys->p_input, "length", &val);
-        sprintf( length, "%d" , (int)(val.i_time / 1000000) );
+        sprintf( length, "%"PRIi64, (int64_t)val.i_time / INT64_C(1000000) );
 
         var_Get( p_sys->p_input, "state", &val );
         if( val.i_int == PLAYING_S )
         {
-            sprintf( state, "playing" );
+            state = "playing";
+        }
+        else if( val.i_int == OPENING_S )
+        {
+            state = "opening/connecting";
+        }
+        else if( val.i_int == BUFFERING_S )
+        {
+            state = "buffering";
         }
         else if( val.i_int == PAUSE_S )
         {
-            sprintf( state, "paused" );
+            state = "paused";
         }
         else
         {
-            sprintf( state, "stop" );
+            state = "stop";
         }
     }
     else
@@ -536,47 +422,47 @@ static void ParseExecute( httpd_file_sys_t *p_args, char *p_buffer,
         sprintf( position, "%d", 0 );
         sprintf( time, "%d", 0 );
         sprintf( length, "%d", 0 );
-        sprintf( state, "stop" );
+        state = "stop";
     }
-#undef p_sys
 
     aout_VolumeGet( p_args->p_intf, &i_volume );
     sprintf( volume, "%d", (int)i_volume );
 
-    p_args->vars = E_(mvar_New)( "variables", "" );
-    E_(mvar_AppendNewVar)( p_args->vars, "url_param",
+    p_args->vars = mvar_New( "variables", "" );
+    mvar_AppendNewVar( p_args->vars, "url_param",
                            i_request > 0 ? "1" : "0" );
-    E_(mvar_AppendNewVar)( p_args->vars, "url_value", p_request );
-    E_(mvar_AppendNewVar)( p_args->vars, "version", VLC_Version() );
-    E_(mvar_AppendNewVar)( p_args->vars, "copyright", COPYRIGHT_MESSAGE );
-    E_(mvar_AppendNewVar)( p_args->vars, "vlc_compile_by", VLC_CompileBy() );
-    E_(mvar_AppendNewVar)( p_args->vars, "vlc_compile_host",
+    mvar_AppendNewVar( p_args->vars, "url_value", p_request );
+    mvar_AppendNewVar( p_args->vars, "version", VLC_Version() );
+    mvar_AppendNewVar( p_args->vars, "copyright", COPYRIGHT_MESSAGE );
+    mvar_AppendNewVar( p_args->vars, "vlc_compile_by", VLC_CompileBy() );
+    mvar_AppendNewVar( p_args->vars, "vlc_compile_host",
                            VLC_CompileHost() );
-    E_(mvar_AppendNewVar)( p_args->vars, "vlc_compile_domain",
+    mvar_AppendNewVar( p_args->vars, "vlc_compile_domain",
                            VLC_CompileDomain() );
-    E_(mvar_AppendNewVar)( p_args->vars, "vlc_compiler", VLC_Compiler() );
-#ifndef HAVE_SHARED_LIBVLC
-    E_(mvar_AppendNewVar)( p_args->vars, "vlc_changeset", VLC_Changeset() );
-#endif
-    E_(mvar_AppendNewVar)( p_args->vars, "stream_position", position );
-    E_(mvar_AppendNewVar)( p_args->vars, "stream_time", time );
-    E_(mvar_AppendNewVar)( p_args->vars, "stream_length", length );
-    E_(mvar_AppendNewVar)( p_args->vars, "volume", volume );
-    E_(mvar_AppendNewVar)( p_args->vars, "stream_state", state );
-    E_(mvar_AppendNewVar)( p_args->vars, "charset", ((intf_sys_t *)p_args->p_intf->p_sys)->psz_charset );
+    mvar_AppendNewVar( p_args->vars, "vlc_compiler", VLC_Compiler() );
+    mvar_AppendNewVar( p_args->vars, "vlc_changeset", VLC_Changeset() );
+    mvar_AppendNewVar( p_args->vars, "stream_position", position );
+    mvar_AppendNewVar( p_args->vars, "stream_time", time );
+    mvar_AppendNewVar( p_args->vars, "stream_length", length );
+    mvar_AppendNewVar( p_args->vars, "volume", volume );
+    mvar_AppendNewVar( p_args->vars, "stream_state", state );
+    mvar_AppendNewVar( p_args->vars, "charset", "UTF-8" );
 
     /* Stats */
-#define p_sys p_args->p_intf->p_sys
     if( p_sys->p_input )
     {
-        input_item_t *p_item = p_sys->p_input->input.p_item;
+        /* FIXME: Workarround a stupid assert in input_GetItem */
+        input_item_t *p_item = p_sys->p_input && p_sys->p_input->p
+                               ? input_GetItem( p_sys->p_input )
+                               : NULL;
+
         if( p_item )
         {
             vlc_mutex_lock( &p_item->p_stats->lock );
 #define STATS_INT( n ) sprintf( stats, "%d", p_item->p_stats->i_ ## n ); \
-                       E_(mvar_AppendNewVar)( p_args->vars, #n, stats );
+                       mvar_AppendNewVar( p_args->vars, #n, stats );
 #define STATS_FLOAT( n ) sprintf( stats, "%f", p_item->p_stats->f_ ## n ); \
-                       E_(mvar_AppendNewVar)( p_args->vars, #n, stats );
+                       mvar_AppendNewVar( p_args->vars, #n, stats );
             STATS_INT( read_bytes )
             STATS_FLOAT( input_bitrate )
             STATS_INT( demux_read_bytes )
@@ -595,36 +481,40 @@ static void ParseExecute( httpd_file_sys_t *p_args, char *p_buffer,
             vlc_mutex_unlock( &p_item->p_stats->lock );
         }
     }
-#undef p_sys
 
-    E_(SSInit)( &p_args->stack );
+    SSInit( &p_args->stack );
 
     /* allocate output */
     *pi_data = i_buffer + 1000;
     dst = *pp_data = malloc( *pi_data );
 
     /* we parse executing all  <vlc /> macros */
-    E_(Execute)( p_args, p_request, i_request, pp_data, pi_data, &dst,
+    Execute( p_args, p_request, i_request, pp_data, pi_data, &dst,
                  &p_buffer[0], &p_buffer[i_buffer] );
 
     *dst     = '\0';
     *pi_data = dst - *pp_data;
 
-    E_(SSClean)( &p_args->stack );
-    E_(mvar_Delete)( p_args->vars );
+    if( p_sys->p_input != NULL )
+    {
+        vlc_object_release( p_sys->p_input );
+        p_sys->p_input = NULL;
+    }
+    SSClean( &p_args->stack );
+    mvar_Delete( p_args->vars );
 }
 
-int  E_(HttpCallback)( httpd_file_sys_t *p_args,
+int  HttpCallback( httpd_file_sys_t *p_args,
                        httpd_file_t *p_file,
                        uint8_t *_p_request,
                        uint8_t **_pp_data, int *pi_data )
 {
+    VLC_UNUSED(p_file);
     char *p_request = (char *)_p_request;
     char **pp_data = (char **)_pp_data;
     FILE *f;
 
-    /* FIXME: do we need character encoding translation here? */
-    if( ( f = fopen( p_args->file, "r" ) ) == NULL )
+    if( ( f = utf8_fopen( p_args->file, "r" ) ) == NULL )
     {
         Callback404( p_args, pp_data, pi_data );
         return VLC_SUCCESS;
@@ -632,7 +522,7 @@ int  E_(HttpCallback)( httpd_file_sys_t *p_args,
 
     if( !p_args->b_html )
     {
-        E_(FileLoad)( f, pp_data, pi_data );
+        FileLoad( f, pp_data, pi_data );
     }
     else
     {
@@ -640,7 +530,7 @@ int  E_(HttpCallback)( httpd_file_sys_t *p_args,
         char *p_buffer;
 
         /* first we load in a temporary buffer */
-        E_(FileLoad)( f, &p_buffer, &i_buffer );
+        FileLoad( f, &p_buffer, &i_buffer );
 
         ParseExecute( p_args, p_buffer, i_buffer, p_request, pp_data, pi_data );
 
@@ -657,24 +547,25 @@ int  E_(HttpCallback)( httpd_file_sys_t *p_args,
  ****************************************************************************
  * call the external handler and parse vlc macros if Content-Type is HTML
  ****************************************************************************/
-int  E_(HandlerCallback)( httpd_handler_sys_t *p_args,
+int  HandlerCallback( httpd_handler_sys_t *p_args,
                           httpd_handler_t *p_handler, char *_p_url,
                           uint8_t *_p_request, int i_type,
                           uint8_t *_p_in, int i_in,
                           char *psz_remote_addr, char *psz_remote_host,
                           uint8_t **_pp_data, int *pi_data )
 {
+    VLC_UNUSED(p_handler); VLC_UNUSED(_p_in);
     char *p_url = (char *)_p_url;
     char *p_request = (char *)_p_request;
     char **pp_data = (char **)_pp_data;
-    char *p_in = (char *)p_in;
+    char *p_in = (char *)_p_in;
     int i_request = p_request != NULL ? strlen( p_request ) : 0;
     char *p;
     int i_env = 0;
     char **ppsz_env = NULL;
     char *psz_tmp;
     char sep;
-    int  i_buffer;
+    size_t i_buffer;
     char *p_buffer;
     char *psz_cwd, *psz_file = NULL;
     int i_ret;
@@ -845,8 +736,7 @@ int  E_(HandlerCallback)( httpd_handler_sys_t *p_args,
                 NULL );
     TAB_REMOVE( p_args->p_association->i_argc, p_args->p_association->ppsz_argv,
                 psz_file );
-    if( psz_cwd != NULL )
-        free( psz_cwd );
+    free( psz_cwd );
     while( i_env )
         TAB_REMOVE( i_env, ppsz_env, ppsz_env[0] );
 
@@ -883,3 +773,92 @@ int  E_(HandlerCallback)( httpd_handler_sys_t *p_args,
 
     return VLC_SUCCESS;
 }
+
+int  ArtCallback( httpd_handler_sys_t *p_args,
+                          httpd_handler_t *p_handler, char *_p_url,
+                          uint8_t *p_request, int i_type,
+                          uint8_t *p_in, int i_in,
+                          char *psz_remote_addr, char *psz_remote_host,
+                          uint8_t **pp_data, int *pi_data )
+{
+    VLC_UNUSED(p_handler); VLC_UNUSED(_p_url); VLC_UNUSED(i_type); 
+    VLC_UNUSED(p_in); VLC_UNUSED(i_in); VLC_UNUSED(psz_remote_addr); 
+    VLC_UNUSED(psz_remote_host); 
+
+    char *psz_art = NULL;
+    intf_thread_t *p_intf = p_args->file.p_intf;
+    intf_sys_t *p_sys = p_intf->p_sys;
+    char psz_id[16];
+    input_item_t *p_item = NULL;
+    int i_id;
+
+    psz_id[0] = '\0';
+    if( p_request )
+        ExtractURIValue( (char *)p_request, "id", psz_id, 15 );
+    i_id = atoi( psz_id );
+    if( i_id )
+    {
+        playlist_item_t *p_pl_item = playlist_ItemGetById( p_sys->p_playlist,
+                                                           i_id, false );
+        if( p_pl_item )
+            p_item = p_pl_item->p_input;
+    }
+    else
+    {
+        /* FIXME: Workarround a stupid assert in input_GetItem */
+        if( p_sys->p_input && p_sys->p_input->p )
+            p_item = input_GetItem( p_sys->p_input );
+    }
+
+    if( p_item )
+    {
+        psz_art = input_item_GetArtURL( p_item );
+    }
+
+    if( psz_art && !strncmp( psz_art, "file://", strlen( "file://" ) ) )
+    {
+        FILE *f;
+        char *psz_ext;
+        char *psz_header;
+        char *p_data = NULL;
+        int i_header_size, i_data;
+
+        if( ( f = utf8_fopen( psz_art + strlen( "file://" ), "r" ) ) == NULL )
+        {
+            msg_Dbg( p_intf, "Couldn't open album art file %s",
+                     psz_art + strlen( "file://" ) );
+            Callback404( &p_args->file, (char**)pp_data, pi_data );
+            free( psz_art );
+            return VLC_SUCCESS;
+        }
+
+        FileLoad( f, &p_data, &i_data );
+
+        fclose( f );
+
+        psz_ext = strrchr( psz_art, '.' );
+        if( psz_ext ) psz_ext++;
+
+#define HEADER  "Content-Type: image/%s\n" \
+                "Content-Length: %d\n" \
+                "\n"
+        i_header_size = asprintf( &psz_header, HEADER, psz_ext, i_data );
+#undef HEADER
+
+        *pi_data = i_header_size + i_data;
+        *pp_data = (uint8_t*)malloc( *pi_data );
+        memcpy( *pp_data, psz_header, i_header_size );
+        memcpy( *pp_data+i_header_size, p_data, i_data );
+        free( psz_header );
+        free( p_data );
+    }
+    else
+    {
+        msg_Dbg( p_intf, "No album art found" );
+        Callback404( &p_args->file, (char**)pp_data, pi_data );
+    }
+
+    free( psz_art );
+
+    return VLC_SUCCESS;
+}
