@@ -2,7 +2,7 @@
  * subtitle.c: Demux vobsub files.
  *****************************************************************************
  * Copyright (C) 1999-2004 the VideoLAN team
- * $Id: vobsub.c 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: 508a42cbd68994286ea573522551e91bfaac9c20 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Derk-Jan Hartman <hartman at videolan dot org>
@@ -19,20 +19,24 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
 
 #include <errno.h>
 #include <sys/types.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>
-#include "vlc_video.h"
+#include <vlc_demux.h>
+#include <vlc_charset.h>
 
 #include "ps.h"
 
@@ -45,11 +49,11 @@ static int  Open ( vlc_object_t *p_this );
 static void Close( vlc_object_t *p_this );
 
 vlc_module_begin();
-    set_description( _("Vobsub subtitles demux") );
+    set_description( N_("Vobsub subtitles parser") );
     set_category( CAT_INPUT );
     set_subcategory( SUBCAT_INPUT_DEMUX );
-    set_capability( "demux2", 1 );
-    
+    set_capability( "demux", 1 );
+
     set_callbacks( Open, Close );
 
     add_shortcut( "vobsub" );
@@ -80,10 +84,12 @@ typedef struct
     es_format_t fmt;
     es_out_id_t *p_es;
     int         i_track_id;
-    
+
     int         i_current_subtitle;
     int         i_subtitles;
     subtitle_t  *p_subtitles;
+
+    int64_t     i_delay;
 } vobsub_track_t;
 
 struct demux_sys_t
@@ -92,14 +98,16 @@ struct demux_sys_t
     int64_t     i_length;
 
     text_t      txt;
-    FILE        *p_vobsub_file;
-    
+    stream_t    *p_vobsub_stream;
+
     /* all tracks */
     int            i_tracks;
     vobsub_track_t *track;
-    
+
     int         i_original_frame_width;
     int         i_original_frame_height;
+    bool  b_palette;
+    uint32_t    palette[16];
 };
 
 static int Demux( demux_t * );
@@ -143,11 +151,13 @@ static int Open ( vlc_object_t *p_this )
     p_demux->pf_control = Control;
     p_demux->p_sys = p_sys = malloc( sizeof( demux_sys_t ) );
     p_sys->i_length = 0;
-    p_sys->p_vobsub_file = NULL;
+    p_sys->p_vobsub_stream = NULL;
     p_sys->i_tracks = 0;
     p_sys->track = (vobsub_track_t *)malloc( sizeof( vobsub_track_t ) );
     p_sys->i_original_frame_width = -1;
     p_sys->i_original_frame_height = -1;
+    p_sys->b_palette = false;
+    memset( p_sys->palette, 0, 16 * sizeof( uint32_t ) );
 
     /* Load the whole file */
     TextLoad( &p_sys->txt, p_demux->s );
@@ -172,18 +182,22 @@ static int Open ( vlc_object_t *p_this )
         }
     }
 
-    i_len = strlen( p_demux->psz_path );
-    psz_vobname = strdup( p_demux->psz_path );
-
-    strcpy( psz_vobname + i_len - 4, ".sub" );
+    if( asprintf( &psz_vobname, "%s://%s", p_demux->psz_access, p_demux->psz_path ) == -1 )
+    {
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    i_len = strlen( psz_vobname );
+    if( i_len >= 4 ) memcpy( psz_vobname + i_len - 4, ".sub", 4 );
 
     /* open file */
-    if( !( p_sys->p_vobsub_file = fopen( psz_vobname, "rb" ) ) )
+    p_sys->p_vobsub_stream = stream_UrlNew( p_demux, psz_vobname );
+    if( p_sys->p_vobsub_stream == NULL )
     {
         msg_Err( p_demux, "couldn't open .sub Vobsub file: %s",
                  psz_vobname );
-        free( p_sys );
         free( psz_vobname );
+        free( p_sys );
         return VLC_EGENERIC;
     }
     free( psz_vobname );
@@ -196,15 +210,18 @@ static int Open ( vlc_object_t *p_this )
  *****************************************************************************/
 static void Close( vlc_object_t *p_this )
 {
+    int i;
     demux_t *p_demux = (demux_t*)p_this;
     demux_sys_t *p_sys = p_demux->p_sys;
 
-    /* Clean all subs from all tracks
-    if( p_sys->subtitle )
-        free( p_sys->subtitle );
-*/
-    if( p_sys->p_vobsub_file )
-        fclose( p_sys->p_vobsub_file );
+    /* Clean all subs from all tracks */
+    for( i = 0; i < p_sys->i_tracks; i++ )
+        free( p_sys->track[i].p_subtitles );
+
+    free( p_sys->track );
+
+    if( p_sys->p_vobsub_stream )
+        stream_Delete( p_sys->p_vobsub_stream );
 
     free( p_sys );
 }
@@ -230,7 +247,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             pi64 = (int64_t*)va_arg( args, int64_t * );
             for( i = 0; i < p_sys->i_tracks; i++ )
             {
-                vlc_bool_t b_selected;
+                bool b_selected;
                 /* Check the ES is selected */
                 es_out_Control( p_demux->out, ES_OUT_GET_ES_STATE,
                                 p_sys->track[i].p_es, &b_selected );
@@ -263,7 +280,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             pf = (double*)va_arg( args, double * );
             for( i = 0; i < p_sys->i_tracks; i++ )
             {
-                vlc_bool_t b_selected;
+                bool b_selected;
                 /* Check the ES is selected */
                 es_out_Control( p_demux->out, ES_OUT_GET_ES_STATE,
                                 p_sys->track[i].p_es, &b_selected );
@@ -323,7 +340,7 @@ static int Demux( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     int64_t i_maxdate;
-    int i;
+    int i, i_read;
 
     for( i = 0; i < p_sys->i_tracks; i++ )
     {
@@ -353,10 +370,10 @@ static int Demux( demux_t *p_demux )
             if( i_size <= 0 ) i_size = 65535;   /* Invalid or EOF */
 
             /* Seek at the right place */
-            if( fseek( p_sys->p_vobsub_file, i_pos, SEEK_SET ) )
+            if( stream_Seek( p_sys->p_vobsub_stream, i_pos ) )
             {
                 msg_Warn( p_demux,
-                          "cannot seek at right vobsub location %d", i_pos );
+                          "cannot seek in the VobSub to the correct time %d", i_pos );
                 tk.i_current_subtitle++;
                 continue;
             }
@@ -369,14 +386,14 @@ static int Demux( demux_t *p_demux )
             }
 
             /* read data */
-            p_block->i_buffer = fread( p_block->p_buffer, 1, i_size,
-                                       p_sys->p_vobsub_file );
-            if( p_block->i_buffer <= 6 )
+            i_read = stream_Read( p_sys->p_vobsub_stream, p_block->p_buffer, i_size );
+            if( i_read <= 6 )
             {
                 block_Release( p_block );
                 tk.i_current_subtitle++;
                 continue;
             }
+            p_block->i_buffer = i_read;
 
             /* pts */
             p_block->i_pts = tk.p_subtitles[tk.i_current_subtitle].i_start;
@@ -434,9 +451,8 @@ static void TextUnload( text_t *txt )
     int i;
 
     for( i = 0; i < txt->i_line_count; i++ )
-    {
         free( txt->line[i] );
-    }
+
     free( txt->line );
     txt->i_line       = 0;
     txt->i_line_count = 0;
@@ -463,8 +479,8 @@ static int ParseVobSubIDX( demux_t *p_demux )
         {
             return( VLC_EGENERIC );
         }
-        
-        if( *line == 0 || *line == '\r' || *line == '\n' || *line == '#' ) 
+
+        if( *line == 0 || *line == '\r' || *line == '\n' || *line == '#' )
             continue;
         else if( !strncmp( "size:", line, 5 ) )
         {
@@ -479,6 +495,43 @@ static int ParseVobSubIDX( demux_t *p_demux )
                 msg_Warn( p_demux, "reading original frame size failed" );
             }
         }
+        else if( !strncmp( "palette:", line, 8 ) )
+        {
+            int i;
+
+            /* Store the palette of the subs */
+            if( sscanf( line, "palette: %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x, %x",
+                        &p_sys->palette[0], &p_sys->palette[1], &p_sys->palette[2], &p_sys->palette[3],
+                        &p_sys->palette[4], &p_sys->palette[5], &p_sys->palette[6], &p_sys->palette[7],
+                        &p_sys->palette[8], &p_sys->palette[9], &p_sys->palette[10], &p_sys->palette[11],
+                        &p_sys->palette[12], &p_sys->palette[13], &p_sys->palette[14], &p_sys->palette[15] ) == 16 )
+            {
+                for( i = 0; i < 16; i++ )
+                {
+                    uint8_t r = 0, g = 0, b = 0;
+                    uint8_t y = 0, u = 0, v = 0;
+                    r = (p_sys->palette[i] >> 16) & 0xff;
+                    g = (p_sys->palette[i] >> 8) & 0xff;
+                    b = (p_sys->palette[i] >> 0) & 0xff;
+                    /* msg_Dbg( p_demux, "palette %d: R=%x, G=%x, B=%x", i, r, g, b ); */
+                    y = (uint8_t) __MIN(abs(r * 2104 + g * 4130 + b * 802 + 4096 + 131072) >> 13, 235);
+                    u = (uint8_t) __MIN(abs(r * -1214 + g * -2384 + b * 3598 + 4096 + 1048576) >> 13, 240);
+                    v = (uint8_t) __MIN(abs(r * 3598 + g * -3013 + b * -585 + 4096 + 1048576) >> 13, 240);
+                    p_sys->palette[i] = 0;
+                    p_sys->palette[i] |= (y&0xff)<<16;
+                    p_sys->palette[i] |= (u&0xff);
+                    p_sys->palette[i] |= (v&0xff)<<8;
+                    /* msg_Dbg( p_demux, "palette %d: y=%x, u=%x, v=%x", i, y, u, v ); */
+
+                }
+                p_sys->b_palette = true;
+                msg_Dbg( p_demux, "vobsub palette read" );
+            }
+            else
+            {
+                msg_Warn( p_demux, "reading original palette failed" );
+            }
+        }
         else if( !strncmp( "id:", line, 3 ) )
         {
             char language[20];
@@ -499,13 +552,19 @@ static int ParseVobSubIDX( demux_t *p_demux )
                 current_tk->i_subtitles = 0;
                 current_tk->p_subtitles = (subtitle_t*)malloc( sizeof( subtitle_t ) );;
                 current_tk->i_track_id = i_track_id;
+                current_tk->i_delay = (int64_t)0;
 
                 es_format_Init( &fmt, SPU_ES, VLC_FOURCC( 's','p','u',' ' ) );
                 fmt.subs.spu.i_original_frame_width = p_sys->i_original_frame_width;
                 fmt.subs.spu.i_original_frame_height = p_sys->i_original_frame_height;
                 fmt.psz_language = strdup( language );
-                current_tk->p_es = es_out_Add( p_demux->out, &fmt );
+                if( p_sys->b_palette )
+                {
+                    fmt.subs.spu.palette[0] = 0xBeef;
+                    memcpy( &fmt.subs.spu.palette[1], p_sys->palette, 16 * sizeof( uint32_t ) );
+                }
 
+                current_tk->p_es = es_out_Add( p_demux->out, &fmt );
                 msg_Dbg( p_demux, "new vobsub track detected" );
             }
             else
@@ -516,34 +575,70 @@ static int ParseVobSubIDX( demux_t *p_demux )
         else if( !strncmp( line, "timestamp:", 10 ) )
         {
             /*
-             * timestamp: hh:mm:ss:mss, filepos: loc
+             * timestamp: [sign]hh:mm:ss:mss, filepos: loc
              * loc is the hex location of the spu in the .sub file
-             *
              */
-            int h, m, s, ms, loc;
+            int h, m, s, ms, count, loc = 0;
+            int i_sign = 1;
             int64_t i_start, i_location = 0;
-            
+
             vobsub_track_t *current_tk = &p_sys->track[p_sys->i_tracks - 1];
 
-            if( sscanf( line, "timestamp: %d:%d:%d:%d, filepos: %x",
-                        &h, &m, &s, &ms, &loc ) == 5 )
+            if( sscanf( line, "timestamp: %d%n:%d:%d:%d, filepos: %x",
+                        &h, &count, &m, &s, &ms, &loc ) >= 5  )
             {
                 subtitle_t *current_sub;
-                
+
+                if( line[count-3] == '-' )
+                {
+                    i_sign = -1;
+                    h = -h;
+                }
                 i_start = (int64_t) ( h * 3600*1000 +
                             m * 60*1000 +
                             s * 1000 +
                             ms ) * 1000;
                 i_location = loc;
-                
+
                 current_tk->i_subtitles++;
                 current_tk->p_subtitles = (subtitle_t*)realloc( current_tk->p_subtitles, sizeof( subtitle_t ) * (current_tk->i_subtitles + 1 ) );
                 current_sub = &current_tk->p_subtitles[current_tk->i_subtitles - 1];
-                
-                current_sub->i_start = (int64_t) i_start;
+
+                current_sub->i_start = (int64_t) i_start * i_sign;
+                current_sub->i_start += current_tk->i_delay;
                 current_sub->i_vobsub_location = i_location;
             }
         }
+        else if( !strncasecmp( line, "delay:", 6 ) )
+        {
+            /*
+             * delay: [sign]hh:mm:ss:mss
+             */
+            int h, m, s, ms, count = 0;
+            int i_sign = 1;
+            int64_t i_gap = 0;
+
+            vobsub_track_t *current_tk = &p_sys->track[p_sys->i_tracks - 1];
+
+            if( sscanf( line, "%*celay: %d%n:%d:%d:%d",
+                        &h, &count, &m, &s, &ms ) >= 4 )
+            {
+                if( line[count-3] == '-' )
+                {
+                    i_sign = -1;
+                    h = -h;
+                }
+                i_gap = (int64_t) ( h * 3600*1000 +
+                            m * 60*1000 +
+                            s * 1000 +
+                            ms ) * 1000;
+
+                current_tk->i_delay = current_tk->i_delay + (i_gap * i_sign);
+                msg_Dbg( p_demux, "sign: %+d gap: %+lld global delay: %+lld",
+                         i_sign, (long long)i_gap,
+                         (long long)current_tk->i_delay  );
+            }
+        }
     }
     return( 0 );
 }
@@ -599,7 +694,7 @@ static int DemuxVobSub( demux_t *p_demux, block_t *p_bk )
 #define tk p_sys->track[i]
             p_pkt->i_dts = p_pkt->i_pts = p_bk->i_pts;
             p_pkt->i_length = 0;
-            
+
             if( tk.p_es && tk.i_track_id == i_spu )
             {
                 es_out_Send( p_demux->out, tk.p_es, p_pkt );
