@@ -1,18 +1,19 @@
 /*****************************************************************************
  * controls.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2005 the VideoLAN team
- * $Id: controls.h 15491 2006-04-30 21:09:16Z bigben $
+ * Copyright (C) 2002-2007 the VideoLAN team
+ * $Id: e077716b354df8d241027575620dca2a097b7f0f $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
  *          Derk-Jan Hartman <thedj@users.sourceforge.net>
+ *          Felix Kühne <fkuehne at videolan org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -23,16 +24,25 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#import "fspanel.h"
+
 /*****************************************************************************
- * VLCControls interface 
+ * VLCControls interface
  *****************************************************************************/
 @interface VLCControls : NSObject
 {
     IBOutlet id o_main;
 
-    IBOutlet id o_btn_fullscreen;
     IBOutlet id o_volumeslider;
 
+    IBOutlet id o_btn_shuffle;
+    IBOutlet id o_btn_addNode;
+    IBOutlet id o_btn_repeat;
+    
+    NSImage * o_repeat_single;
+    NSImage * o_repeat_all;
+    NSImage * o_repeat_off;
+
     IBOutlet id o_specificTime_cancel_btn;
     IBOutlet id o_specificTime_enter_fld;
     IBOutlet id o_specificTime_goTo_lbl;
@@ -41,7 +51,10 @@
     IBOutlet id o_specificTime_sec_lbl;
     IBOutlet id o_specificTime_stepper;
     IBOutlet id o_specificTime_mi;
+
+    VLCFSPanel *o_fs_panel;
 }
+- (void)controlTintChanged;
 
 - (IBAction)play:(id)sender;
 - (IBAction)stop:(id)sender;
@@ -53,36 +66,51 @@
 - (IBAction)random:(id)sender;
 - (IBAction)repeat:(id)sender;
 - (IBAction)loop:(id)sender;
+- (IBAction)repeatButtonAction:(id)sender;
+
+/* the three ugly helpers again */
+- (void)repeatOne;
+- (void)repeatAll;
+- (void)repeatOff;
+- (void)shuffle;
 
 - (IBAction)forward:(id)sender;
 - (IBAction)backward:(id)sender;
 
+- (id)getVoutView;
+
 - (IBAction)volumeUp:(id)sender;
 - (IBAction)volumeDown:(id)sender;
 - (IBAction)mute:(id)sender;
 - (IBAction)volumeSliderUpdated:(id)sender;
 
+- (IBAction)showPosition: (id)sender;
+- (IBAction)toogleFullscreen:(id)sender;
+- (BOOL) isFullscreen;
 - (IBAction)windowAction:(id)sender;
 - (BOOL)keyEvent:(NSEvent *)o_event;
+- (void)scrollWheel: (NSEvent *)theEvent;
 
 - (void)setupVarMenuItem:(NSMenuItem *)o_mi
-                    target:(vlc_object_t *)p_object
-                    var:(const char *)psz_variable
-                    selector:(SEL)pf_callback;
+                  target:(vlc_object_t *)p_object
+                     var:(const char *)psz_variable
+                selector:(SEL)pf_callback;
 - (void)setupVarMenu:(NSMenu *)o_menu
-                    forMenuItem: (NSMenuItem *)o_parent
-                    target:(vlc_object_t *)p_object
-                    var:(const char *)psz_variable
-                    selector:(SEL)pf_callback;
+         forMenuItem: (NSMenuItem *)o_parent
+              target:(vlc_object_t *)p_object
+                 var:(const char *)psz_variable
+            selector:(SEL)pf_callback;
 - (IBAction)toggleVar:(id)sender;
 - (int)toggleVarThread:(id)_o_data;
 
 - (IBAction)goToSpecificTime:(id)sender;
 
+- (id)getFSPanel;
+
 @end
 
 /*****************************************************************************
- * VLCMenuExt interface 
+ * VLCMenuExt interface
  *****************************************************************************
  * This holds our data for autogenerated menus
  *****************************************************************************/
@@ -94,8 +122,10 @@
     int i_type;
 }
 
-- (id)initWithVar: (const char *)_psz_name Object: (int)i_id
-        Value: (vlc_value_t)val ofType: (int)_i_type;
+- (id)initWithVar: (const char *)_psz_name 
+           Object: (int)i_id
+            Value: (vlc_value_t)val 
+           ofType: (int)_i_type;
 - (char *)name;
 - (int)objectID;
 - (vlc_value_t)value;
@@ -104,7 +134,7 @@
 @end
 
 /*****************************************************************************
- * VLCTimeField interface 
+ * VLCTimeField interface
  *****************************************************************************
  * we need the implementation to catch our click-event in the controller window
  *****************************************************************************/
