@@ -1,8 +1,8 @@
 /*****************************************************************************
  * playlist.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2005 the VideoLAN team
- * $Id: playlist.h 14335 2006-02-16 20:05:18Z fkuehne $
+ * Copyright (C) 2002-2006 the VideoLAN team
+ * $Id: 698bb6bc7604f67a1f53c5cbbda4e82f625b9a04 $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <hartman at videolan dot org>
@@ -11,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -42,7 +42,6 @@
     IBOutlet id o_outline_view;
 
     NSMutableDictionary *o_outline_dict;
-    int i_current_view;
 }
 
 - (void)initStrings;
@@ -74,8 +73,6 @@
     IBOutlet id o_playlist_view;
     IBOutlet id o_status_field;
     IBOutlet id o_search_field;
-    IBOutlet id o_random_ckb;
-    IBOutlet id o_loop_popup;
     IBOutlet id o_mi_save_playlist;
     IBOutlet id o_ctx_menu;
 
@@ -96,6 +93,11 @@
     IBOutlet id o_mm_mi_services;
     IBOutlet id o_mm_mu_services;
 
+    IBOutlet id o_save_accessory_view;
+    IBOutlet id o_save_accessory_popup;
+    IBOutlet id o_save_accessory_text;
+
+
     NSImage *o_descendingSortingImage;
     NSImage *o_ascendingSortingImage;
 
@@ -105,7 +107,7 @@
     BOOL b_selected_item_met;
     BOOL b_isSortDescending;
     id o_tc_sortColumn;
-    
+
     /* "add node" button and menu entry */
     IBOutlet id o_mi_addNode;
     IBOutlet id o_btn_addNode;
@@ -114,7 +116,6 @@
 - (void)searchfieldChanged:(NSNotification *)o_notification;
 - (NSMenu *)menuForEvent:(NSEvent *)o_event;
 
-- (IBAction)handlePopUp:(id)sender;
 - (IBAction)searchItem:(id)sender;
 
 - (void)playlistUpdated;
@@ -122,6 +123,8 @@
 - (void)sortNode:(int)i_mode;
 - (void)updateRowSelection;
 
+- (BOOL)isSelectionEmpty;
+
 - (IBAction)servicesChange:(id)sender;
 - (IBAction)playItem:(id)sender;
 - (IBAction)preparseItem:(id)sender;
@@ -133,10 +136,9 @@
 - (IBAction)recursiveExpandNode:(id)sender;
 
 - (IBAction)addNode:(id)sender;
+- (void)addNodeThreadedly;
 
 - (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue;
-- (void)appendNodeArray:(NSArray*)o_array inNode:(playlist_item_t *)p_node atPos:(int)i_position inView:(int)i_view enqueue:(BOOL)b_enqueue;
-
+- (void)appendNodeArray:(NSArray*)o_array inNode:(playlist_item_t *)p_node atPos:(int)i_position enqueue:(BOOL)b_enqueue;
 
 @end
-
