@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input.c : internal management of input streams for the audio output
  *****************************************************************************
- * Copyright (C) 2002-2004 the VideoLAN team
- * $Id: input.c 14953 2006-03-28 20:29:28Z zorglub $
+ * Copyright (C) 2002-2007 the VideoLAN team
+ * $Id: 2e2a4d39d68accddd5c693d65acf0ec0831c414d $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -24,25 +24,44 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>                            /* calloc(), malloc(), free() */
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>                 /* for input_thread_t and i_pts_delay */
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+
+#include <stdio.h>
+#include <string.h>
+#include <math.h>
+#include <assert.h>
+
+#include <vlc_input.h>                 /* for input_thread_t and i_pts_delay */
 
 #ifdef HAVE_ALLOCA_H
 #   include <alloca.h>
 #endif
+#include <vlc_aout.h>
 
-#include "audio_output.h"
 #include "aout_internal.h"
 
-static void inputFailure( aout_instance_t *, aout_input_t *, char * );
+/** FIXME: Ugly but needed to access the counters */
+#include "input/input_internal.h"
+
+#define AOUT_ASSERT_MIXER_LOCKED vlc_assert_locked( &p_aout->mixer_lock )
+#define AOUT_ASSERT_INPUT_LOCKED vlc_assert_locked( &p_input->lock )
+
+static void inputFailure( aout_instance_t *, aout_input_t *, const char * );
+static void inputDrop( aout_instance_t *, aout_input_t *, aout_buffer_t * );
+static void inputResamplingStop( aout_input_t *p_input );
+
 static int VisualizationCallback( vlc_object_t *, char const *,
                                   vlc_value_t, vlc_value_t, void * );
 static int EqualizerCallback( vlc_object_t *, char const *,
                               vlc_value_t, vlc_value_t, void * );
-
+static int ReplayGainCallback( vlc_object_t *, char const *,
+                               vlc_value_t, vlc_value_t, void * );
+static void ReplayGainSelect( aout_instance_t *, aout_input_t * );
 /*****************************************************************************
  * aout_InputNew : allocate a new input and rework the filter pipeline
  *****************************************************************************/
@@ -56,7 +75,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
 
     aout_FormatPrint( p_aout, "input", &p_input->input );
 
-    p_input->i_nb_filters = 0;
+    p_input->i_nb_resamplers = p_input->i_nb_filters = 0;
 
     /* Prepare FIFO. */
     aout_FifoInit( p_aout, &p_input->fifo, p_aout->mixer.mixer.i_rate );
@@ -73,39 +92,38 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
     /* Now add user filters */
     if( var_Type( p_aout, "visual" ) == 0 )
     {
-        module_t *p_module;
         var_Create( p_aout, "visual", VLC_VAR_STRING | VLC_VAR_HASCHOICE );
         text.psz_string = _("Visualizations");
         var_Change( p_aout, "visual", VLC_VAR_SETTEXT, &text, NULL );
-        val.psz_string = ""; text.psz_string = _("Disable");
+        val.psz_string = (char*)""; text.psz_string = _("Disable");
         var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
-        val.psz_string = "spectrometer"; text.psz_string = _("Spectrometer");
+        val.psz_string = (char*)"spectrometer"; text.psz_string = _("Spectrometer");
         var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
-        val.psz_string = "scope"; text.psz_string = _("Scope");
+        val.psz_string = (char*)"scope"; text.psz_string = _("Scope");
         var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
-        val.psz_string = "spectrum"; text.psz_string = _("Spectrum");
+        val.psz_string = (char*)"spectrum"; text.psz_string = _("Spectrum");
+        var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
+        val.psz_string = (char*)"vuMeter"; text.psz_string = _("Vu meter");
         var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
 
         /* Look for goom plugin */
-        p_module = config_FindModule( VLC_OBJECT(p_aout), "goom" );
-        if( p_module )
+        if( module_Exists( VLC_OBJECT(p_aout), "goom" ) )
         {
-            val.psz_string = "goom"; text.psz_string = "Goom";
+            val.psz_string = (char*)"goom"; text.psz_string = (char*)"Goom";
             var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
         }
 
         /* Look for galaktos plugin */
-        p_module = config_FindModule( VLC_OBJECT(p_aout), "galaktos" );
-        if( p_module )
+        if( module_Exists( VLC_OBJECT(p_aout), "galaktos" ) )
         {
-            val.psz_string = "galaktos"; text.psz_string = "GaLaktos";
+            val.psz_string = (char*)"galaktos"; text.psz_string = (char*)"GaLaktos";
             var_Change( p_aout, "visual", VLC_VAR_ADDCHOICE, &val, &text );
         }
 
         if( var_Get( p_aout, "effect-list", &val ) == VLC_SUCCESS )
         {
             var_Set( p_aout, "visual", val );
-            if( val.psz_string ) free( val.psz_string );
+            free( val.psz_string );
         }
         var_AddCallback( p_aout, "visual", VisualizationCallback, NULL );
     }
@@ -123,13 +141,13 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
             text.psz_string = _("Equalizer");
             var_Change( p_aout, "equalizer", VLC_VAR_SETTEXT, &text, NULL );
 
-            val.psz_string = ""; text.psz_string = _("Disable");
+            val.psz_string = (char*)""; text.psz_string = _("Disable");
             var_Change( p_aout, "equalizer", VLC_VAR_ADDCHOICE, &val, &text );
 
             for( i = 0; i < p_config->i_list; i++ )
             {
-                val.psz_string = p_config->ppsz_list[i];
-                text.psz_string = p_config->ppsz_list_text[i];
+                val.psz_string = (char *)p_config->ppsz_list[i];
+                text.psz_string = (char *)p_config->ppsz_list_text[i];
                 var_Change( p_aout, "equalizer", VLC_VAR_ADDCHOICE,
                             &val, &text );
             }
@@ -153,13 +171,54 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
         var_Change( p_aout, "audio-visual", VLC_VAR_SETTEXT, &text, NULL );
     }
 
+    if( var_Type( p_aout, "audio-replay-gain-mode" ) == 0 )
+    {
+        module_config_t *p_config;
+        int i;
+
+        p_config = config_FindConfig( VLC_OBJECT(p_aout), "audio-replay-gain-mode" );
+        if( p_config && p_config->i_list )
+        {
+            var_Create( p_aout, "audio-replay-gain-mode",
+                        VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+
+            text.psz_string = _("Replay gain");
+            var_Change( p_aout, "audio-replay-gain-mode", VLC_VAR_SETTEXT, &text, NULL );
+
+            for( i = 0; i < p_config->i_list; i++ )
+            {
+                val.psz_string = (char *)p_config->ppsz_list[i];
+                text.psz_string = (char *)p_config->ppsz_list_text[i];
+                var_Change( p_aout, "audio-replay-gain-mode", VLC_VAR_ADDCHOICE,
+                            &val, &text );
+            }
+
+            var_AddCallback( p_aout, "audio-replay-gain-mode", ReplayGainCallback, NULL );
+        }
+    }
+    if( var_Type( p_aout, "audio-replay-gain-preamp" ) == 0 )
+    {
+        var_Create( p_aout, "audio-replay-gain-preamp",
+                    VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+    }
+    if( var_Type( p_aout, "audio-replay-gain-default" ) == 0 )
+    {
+        var_Create( p_aout, "audio-replay-gain-default",
+                    VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+    }
+    if( var_Type( p_aout, "audio-replay-gain-peak-protection" ) == 0 )
+    {
+        var_Create( p_aout, "audio-replay-gain-peak-protection",
+                    VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    }
+
     var_Get( p_aout, "audio-filter", &val );
     psz_filters = val.psz_string;
     var_Get( p_aout, "audio-visual", &val );
     psz_visual = val.psz_string;
 
     /* parse user filter lists */
-    for( i_visual = 0; i_visual < 2; i_visual++ )
+    for( i_visual = 0; i_visual < 2 && !AOUT_FMT_NON_LINEAR(&chain_output_format); i_visual++ )
     {
         char *psz_next = NULL;
         char *psz_parser = i_visual ? psz_visual : psz_filters;
@@ -191,7 +250,9 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
             }
 
             /* Create a VLC object */
-            p_filter = vlc_object_create( p_aout, sizeof(aout_filter_t) );
+            static const char typename[] = "audio filter";
+            p_filter = vlc_custom_create( p_aout, sizeof(*p_filter),
+                                          VLC_OBJECT_GENERIC, typename );
             if( p_filter == NULL )
             {
                 msg_Err( p_aout, "cannot add user filter %s (skipped)",
@@ -212,7 +273,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                         sizeof(audio_sample_format_t) );
 
                 p_filter->p_module = module_Need( p_filter, "visualization",
-                                                  psz_parser, VLC_TRUE );
+                                                  psz_parser, true );
             }
             else /* this can be a audio filter module as well as a visualization module */
             {
@@ -223,7 +284,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                         sizeof(audio_sample_format_t) );
 
                 p_filter->p_module = module_Need( p_filter, "audio filter",
-                                              psz_parser, VLC_TRUE );
+                                              psz_parser, true );
 
                 if ( p_filter->p_module == NULL )
                 {
@@ -237,7 +298,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                         aout_FormatPrepare( &p_filter->output );
                         p_filter->p_module = module_Need( p_filter,
                                                           "audio filter",
-                                                          psz_parser, VLC_TRUE );
+                                                          psz_parser, true );
                     }
                     /* try visual filters */
                     else
@@ -248,7 +309,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                                 sizeof(audio_sample_format_t) );
                         p_filter->p_module = module_Need( p_filter,
                                                           "visualization",
-                                                          psz_parser, VLC_TRUE );
+                                                          psz_parser, true );
                     }
                 }
             }
@@ -260,7 +321,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                          psz_parser );
 
                 vlc_object_detach( p_filter );
-                vlc_object_destroy( p_filter );
+                vlc_object_release( p_filter );
 
                 psz_parser = psz_next;
                 continue;
@@ -279,7 +340,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
 
                     module_Unneed( p_filter, p_filter->p_module );
                     vlc_object_detach( p_filter );
-                    vlc_object_destroy( p_filter );
+                    vlc_object_release( p_filter );
 
                     psz_parser = psz_next;
                     continue;
@@ -287,7 +348,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
             }
 
             /* success */
-            p_filter->b_continuity = VLC_FALSE;
+            p_filter->b_continuity = false;
             p_input->pp_filters[p_input->i_nb_filters++] = p_filter;
             memcpy( &chain_input_format, &p_filter->output,
                     sizeof( audio_sample_format_t ) );
@@ -296,8 +357,8 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
             psz_parser = psz_next;
         }
     }
-    if( psz_filters ) free( psz_filters );
-    if( psz_visual ) free( psz_visual );
+    free( psz_filters );
+    free( psz_visual );
 
     /* complete the filter chain if necessary */
     if ( !AOUT_FMTS_IDENTICAL( &chain_input_format, &chain_output_format ) )
@@ -317,11 +378,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
     p_input->input_alloc.i_bytes_per_sec = -1;
 
     /* Create resamplers. */
-    if ( AOUT_FMT_NON_LINEAR( &p_aout->mixer.mixer ) )
-    {
-        p_input->i_nb_resamplers = 0;
-    }
-    else
+    if ( !AOUT_FMT_NON_LINEAR( &p_aout->mixer.mixer ) )
     {
         chain_output_format.i_rate = (__MAX(p_input->input.i_rate,
                                             p_aout->mixer.mixer.i_rate)
@@ -331,7 +388,6 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
             /* Just in case... */
             chain_output_format.i_rate++;
         }
-        p_input->i_nb_resamplers = 0;
         if ( aout_FiltersCreatePipeline( p_aout, p_input->pp_resamplers,
                                          &p_input->i_nb_resamplers,
                                          &chain_output_format,
@@ -351,6 +407,21 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
     }
     p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
 
+    p_input->p_playback_rate_filter = NULL;
+    for( int i = 0; i < p_input->i_nb_filters; i++ )
+    {
+        aout_filter_t *p_filter = p_input->pp_filters[i];
+        if( strcmp( "scaletempo", p_filter->psz_object_name ) == 0 )
+        {
+          p_input->p_playback_rate_filter = p_filter;
+          break;
+        }
+    }
+    if( ! p_input->p_playback_rate_filter && p_input->i_nb_resamplers > 0 )
+    {
+        p_input->p_playback_rate_filter = p_input->pp_resamplers[0];
+    }
+
     aout_FiltersHintBuffers( p_aout, p_input->pp_filters,
                              p_input->i_nb_filters,
                              &p_input->input_alloc );
@@ -363,9 +434,12 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
                                      * p_input->input.i_rate
                                      / p_input->input.i_frame_length) );
 
+    ReplayGainSelect( p_aout, p_input );
+
     /* Success */
-    p_input->b_error = VLC_FALSE;
-    p_input->b_restart = VLC_FALSE;
+    p_input->b_error = false;
+    p_input->b_restart = false;
+    p_input->i_last_input_rate = INPUT_RATE_DEFAULT;
 
     return 0;
 }
@@ -377,6 +451,7 @@ int aout_InputNew( aout_instance_t * p_aout, aout_input_t * p_input )
  *****************************************************************************/
 int aout_InputDelete( aout_instance_t * p_aout, aout_input_t * p_input )
 {
+    AOUT_ASSERT_MIXER_LOCKED;
     if ( p_input->b_error ) return 0;
 
     aout_FiltersDestroyPipeline( p_aout, p_input->pp_filters,
@@ -395,17 +470,21 @@ int aout_InputDelete( aout_instance_t * p_aout, aout_input_t * p_input )
  *****************************************************************************
  * This function must be entered with the input lock.
  *****************************************************************************/
+/* XXX Do not activate it !! */
+//#define AOUT_PROCESS_BEFORE_CHEKS
 int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
-                    aout_buffer_t * p_buffer )
+                    aout_buffer_t * p_buffer, int i_input_rate )
 {
     mtime_t start_date;
+    AOUT_ASSERT_INPUT_LOCKED;
 
     if( p_input->b_restart )
     {
         aout_fifo_t fifo, dummy_fifo;
-        byte_t      *p_first_byte_to_mix;
+        uint8_t     *p_first_byte_to_mix;
 
-        vlc_mutex_lock( &p_aout->mixer_lock );
+        aout_lock_mixer( p_aout );
+        aout_lock_input_fifos( p_aout );
 
         /* A little trick to avoid loosing our input fifo */
         aout_FifoInit( p_aout, &dummy_fifo, p_aout->mixer.mixer.i_rate );
@@ -417,105 +496,117 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
         p_input->p_first_byte_to_mix = p_first_byte_to_mix;
         p_input->fifo = fifo;
 
-        vlc_mutex_unlock( &p_aout->mixer_lock );
+        aout_unlock_input_fifos( p_aout );
+        aout_unlock_mixer( p_aout );
+    }
+
+    if( i_input_rate != INPUT_RATE_DEFAULT && p_input->p_playback_rate_filter == NULL )
+    {
+        inputDrop( p_aout, p_input, p_buffer );
+        return 0;
+    }
+
+#ifdef AOUT_PROCESS_BEFORE_CHEKS
+    /* Run pre-filters. */
+    aout_FiltersPlay( p_aout, p_input->pp_filters, p_input->i_nb_filters,
+                      &p_buffer );
+
+    /* Actually run the resampler now. */
+    if ( p_input->i_nb_resamplers > 0 )
+    {
+        const mtime_t i_date = p_buffer->start_date;
+        aout_FiltersPlay( p_aout, p_input->pp_resamplers,
+                          p_input->i_nb_resamplers,
+                          &p_buffer );
+    }
+
+    if( p_buffer->i_nb_samples <= 0 )
+    {
+        aout_BufferFree( p_buffer );
+        return 0;
+    }
+#endif
+
+    /* Handle input rate change, but keep drift correction */
+    if( i_input_rate != p_input->i_last_input_rate )
+    {
+        unsigned int * const pi_rate = &p_input->p_playback_rate_filter->input.i_rate;
+#define F(r,ir) ( INPUT_RATE_DEFAULT * (r) / (ir) )
+        const int i_delta = *pi_rate - F(p_input->input.i_rate,p_input->i_last_input_rate);
+        *pi_rate = F(p_input->input.i_rate + i_delta, i_input_rate);
+#undef F
+        p_input->i_last_input_rate = i_input_rate;
     }
 
     /* We don't care if someone changes the start date behind our back after
      * this. We'll deal with that when pushing the buffer, and compensate
      * with the next incoming buffer. */
-    vlc_mutex_lock( &p_aout->input_fifos_lock );
+    aout_lock_input_fifos( p_aout );
     start_date = aout_FifoNextStart( p_aout, &p_input->fifo );
-    vlc_mutex_unlock( &p_aout->input_fifos_lock );
+    aout_unlock_input_fifos( p_aout );
 
     if ( start_date != 0 && start_date < mdate() )
     {
         /* The decoder is _very_ late. This can only happen if the user
          * pauses the stream (or if the decoder is buggy, which cannot
          * happen :). */
-        msg_Warn( p_aout, "computed PTS is out of range ("I64Fd"), "
+        msg_Warn( p_aout, "computed PTS is out of range (%"PRId64"), "
                   "clearing out", mdate() - start_date );
-        vlc_mutex_lock( &p_aout->input_fifos_lock );
+        aout_lock_input_fifos( p_aout );
         aout_FifoSet( p_aout, &p_input->fifo, 0 );
         p_input->p_first_byte_to_mix = NULL;
-        vlc_mutex_unlock( &p_aout->input_fifos_lock );
+        aout_unlock_input_fifos( p_aout );
         if ( p_input->i_resampling_type != AOUT_RESAMPLING_NONE )
             msg_Warn( p_aout, "timing screwed, stopping resampling" );
-        p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
-        if ( p_input->i_nb_resamplers != 0 )
-        {
-            p_input->pp_resamplers[0]->input.i_rate = p_input->input.i_rate;
-            p_input->pp_resamplers[0]->b_continuity = VLC_FALSE;
-        }
+        inputResamplingStop( p_input );
         start_date = 0;
-        if( p_input->p_input_thread )
-        {
-            stats_UpdateInteger( p_input->p_input_thread, STATS_LOST_ABUFFERS, 1,
-                                 NULL );
-        }
     }
 
     if ( p_buffer->start_date < mdate() + AOUT_MIN_PREPARE_TIME )
     {
         /* The decoder gives us f*cked up PTS. It's its business, but we
          * can't present it anyway, so drop the buffer. */
-        msg_Warn( p_aout, "PTS is out of range ("I64Fd"), dropping buffer",
+        msg_Warn( p_aout, "PTS is out of range (%"PRId64"), dropping buffer",
                   mdate() - p_buffer->start_date );
-        if( p_input->p_input_thread )
-        {
-            stats_UpdateInteger( p_input->p_input_thread, STATS_LOST_ABUFFERS,
-                                 1, NULL );
-        }
-        aout_BufferFree( p_buffer );
-        p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
-        if ( p_input->i_nb_resamplers != 0 )
-        {
-            p_input->pp_resamplers[0]->input.i_rate = p_input->input.i_rate;
-            p_input->pp_resamplers[0]->b_continuity = VLC_FALSE;
-        }
+
+        inputDrop( p_aout, p_input, p_buffer );
+        inputResamplingStop( p_input );
         return 0;
     }
 
     /* If the audio drift is too big then it's not worth trying to resample
      * the audio. */
+    mtime_t i_pts_tolerance = 3 * AOUT_PTS_TOLERANCE * i_input_rate / INPUT_RATE_DEFAULT;
     if ( start_date != 0 &&
-         ( start_date < p_buffer->start_date - 3 * AOUT_PTS_TOLERANCE ) )
+         ( start_date < p_buffer->start_date - i_pts_tolerance ) )
     {
-        msg_Warn( p_aout, "audio drift is too big ("I64Fd"), clearing out",
+        msg_Warn( p_aout, "audio drift is too big (%"PRId64"), clearing out",
                   start_date - p_buffer->start_date );
-        vlc_mutex_lock( &p_aout->input_fifos_lock );
+        aout_lock_input_fifos( p_aout );
         aout_FifoSet( p_aout, &p_input->fifo, 0 );
         p_input->p_first_byte_to_mix = NULL;
-        vlc_mutex_unlock( &p_aout->input_fifos_lock );
+        aout_unlock_input_fifos( p_aout );
         if ( p_input->i_resampling_type != AOUT_RESAMPLING_NONE )
             msg_Warn( p_aout, "timing screwed, stopping resampling" );
-        p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
-        if ( p_input->i_nb_resamplers != 0 )
-        {
-            p_input->pp_resamplers[0]->input.i_rate = p_input->input.i_rate;
-            p_input->pp_resamplers[0]->b_continuity = VLC_FALSE;
-        }
+        inputResamplingStop( p_input );
         start_date = 0;
     }
     else if ( start_date != 0 &&
-              ( start_date > p_buffer->start_date + 3 * AOUT_PTS_TOLERANCE ) )
+              ( start_date > p_buffer->start_date + i_pts_tolerance) )
     {
-        msg_Warn( p_aout, "audio drift is too big ("I64Fd"), dropping buffer",
+        msg_Warn( p_aout, "audio drift is too big (%"PRId64"), dropping buffer",
                   start_date - p_buffer->start_date );
-        aout_BufferFree( p_buffer );
-        if( p_input->p_input_thread )
-        {
-            stats_UpdateInteger( p_input->p_input_thread, STATS_LOST_ABUFFERS,
-                                 1, NULL );
-        }
+        inputDrop( p_aout, p_input, p_buffer );
         return 0;
     }
 
     if ( start_date == 0 ) start_date = p_buffer->start_date;
 
+#ifndef AOUT_PROCESS_BEFORE_CHEKS
     /* Run pre-filters. */
-
     aout_FiltersPlay( p_aout, p_input->pp_filters, p_input->i_nb_filters,
                       &p_buffer );
+#endif
 
     /* Run the resampler if needed.
      * We first need to calculate the output rate of this resampler. */
@@ -541,7 +632,7 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
         else
             p_input->i_resampling_type = AOUT_RESAMPLING_UP;
 
-        msg_Warn( p_aout, "buffer is "I64Fd" %s, triggering %ssampling",
+        msg_Warn( p_aout, "buffer is %"PRId64" %s, triggering %ssampling",
                           drift > 0 ? drift : -drift,
                           drift > 0 ? "in advance" : "late",
                           drift > 0 ? "down" : "up");
@@ -564,12 +655,15 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
 
         /* Check if everything is back to normal, in which case we can stop the
          * resampling */
-        if( p_input->pp_resamplers[0]->input.i_rate ==
-              p_input->input.i_rate )
+        unsigned int i_nominal_rate =
+          (p_input->pp_resamplers[0] == p_input->p_playback_rate_filter)
+          ? INPUT_RATE_DEFAULT * p_input->input.i_rate / i_input_rate
+          : p_input->input.i_rate;
+        if( p_input->pp_resamplers[0]->input.i_rate == i_nominal_rate )
         {
             p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
-            msg_Warn( p_aout, "resampling stopped after "I64Fi" usec "
-                      "(drift: "I64Fi")",
+            msg_Warn( p_aout, "resampling stopped after %"PRIi64" usec "
+                      "(drift: %"PRIi64")",
                       mdate() - p_input->i_resamp_start_date,
                       p_buffer->start_date - start_date);
         }
@@ -591,16 +685,11 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
             /* If the drift is increasing and not decreasing, than something
              * is bad. We'd better stop the resampling right now. */
             msg_Warn( p_aout, "timing screwed, stopping resampling" );
-            p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
-            p_input->pp_resamplers[0]->input.i_rate = p_input->input.i_rate;
+            inputResamplingStop( p_input );
         }
     }
 
-    /* Adding the start date will be managed by aout_FifoPush(). */
-    p_buffer->end_date = start_date +
-        (p_buffer->end_date - p_buffer->start_date);
-    p_buffer->start_date = start_date;
-
+#ifndef AOUT_PROCESS_BEFORE_CHEKS
     /* Actually run the resampler now. */
     if ( p_input->i_nb_resamplers > 0 )
     {
@@ -609,10 +698,21 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
                           &p_buffer );
     }
 
-    vlc_mutex_lock( &p_aout->input_fifos_lock );
-    aout_FifoPush( p_aout, &p_input->fifo, p_buffer );
-    vlc_mutex_unlock( &p_aout->input_fifos_lock );
+    if( p_buffer->i_nb_samples <= 0 )
+    {
+        aout_BufferFree( p_buffer );
+        return 0;
+    }
+#endif
 
+    /* Adding the start date will be managed by aout_FifoPush(). */
+    p_buffer->end_date = start_date +
+        (p_buffer->end_date - p_buffer->start_date);
+    p_buffer->start_date = start_date;
+
+    aout_lock_input_fifos( p_aout );
+    aout_FifoPush( p_aout, &p_input->fifo, p_buffer );
+    aout_unlock_input_fifos( p_aout );
     return 0;
 }
 
@@ -621,10 +721,10 @@ int aout_InputPlay( aout_instance_t * p_aout, aout_input_t * p_input,
  *****************************************************************************/
 
 static void inputFailure( aout_instance_t * p_aout, aout_input_t * p_input,
-                          char * psz_error_message )
+                          const char * psz_error_message )
 {
     /* error message */
-    msg_Err( p_aout, "couldn't set an input pipeline" );
+    msg_Err( p_aout, "%s", psz_error_message );
 
     /* clean up */
     aout_FiltersDestroyPipeline( p_aout, p_input->pp_filters,
@@ -637,54 +737,45 @@ static void inputFailure( aout_instance_t * p_aout, aout_input_t * p_input,
     var_Destroy( p_aout, "audio-filter" );
     var_Destroy( p_aout, "audio-visual" );
 
+    var_Destroy( p_aout, "audio-replay-gain-mode" );
+    var_Destroy( p_aout, "audio-replay-gain-default" );
+    var_Destroy( p_aout, "audio-replay-gain-preamp" );
+    var_Destroy( p_aout, "audio-replay-gain-peak-protection" );
+
     /* error flag */
     p_input->b_error = 1;
 }
 
-static int ChangeFiltersString( aout_instance_t * p_aout, char* psz_variable,
-                                 char *psz_name, vlc_bool_t b_add )
+static void inputDrop( aout_instance_t *p_aout, aout_input_t *p_input, aout_buffer_t *p_buffer )
 {
-    vlc_value_t val;
-    char *psz_parser;
+    aout_BufferFree( p_buffer );
 
-    var_Get( p_aout, psz_variable, &val );
+    if( !p_input->p_input_thread )
+        return;
 
-    if( !val.psz_string ) val.psz_string = strdup("");
+    vlc_mutex_lock( &p_input->p_input_thread->p->counters.counters_lock);
+    stats_UpdateInteger( p_aout, p_input->p_input_thread->p->counters.p_lost_abuffers, 1, NULL );
+    vlc_mutex_unlock( &p_input->p_input_thread->p->counters.counters_lock);
+}
 
-    psz_parser = strstr( val.psz_string, psz_name );
-
-    if( b_add )
+static void inputResamplingStop( aout_input_t *p_input )
+{
+    p_input->i_resampling_type = AOUT_RESAMPLING_NONE;
+    if( p_input->i_nb_resamplers != 0 )
     {
-        if( !psz_parser )
-        {
-            psz_parser = val.psz_string;
-            asprintf( &val.psz_string, (*val.psz_string) ? "%s:%s" : "%s%s",
-                      val.psz_string, psz_name );
-            free( psz_parser );
-        }
-        else
-        {
-            return 0;
-        }
-    }
-    else
-    {
-        if( psz_parser )
-        {
-            memmove( psz_parser, psz_parser + strlen(psz_name) +
-                     (*(psz_parser + strlen(psz_name)) == ':' ? 1 : 0 ),
-                     strlen(psz_parser + strlen(psz_name)) + 1 );
-        }
-        else
-        {
-            free( val.psz_string );
-            return 0;
-        }
+        p_input->pp_resamplers[0]->input.i_rate =
+            ( p_input->pp_resamplers[0] == p_input->p_playback_rate_filter )
+            ? INPUT_RATE_DEFAULT * p_input->input.i_rate / p_input->i_last_input_rate
+            : p_input->input.i_rate;
+        p_input->pp_resamplers[0]->b_continuity = false;
     }
+}
 
-    var_Set( p_aout, psz_variable, val );
-    free( val.psz_string );
-    return 1;
+static int ChangeFiltersString( aout_instance_t * p_aout, const char* psz_variable,
+                                 const char *psz_name, bool b_add )
+{
+    return AoutChangeFilterString( VLC_OBJECT(p_aout), p_aout,
+                                   psz_variable, psz_name, b_add ) ? 1 : 0;
 }
 
 static int VisualizationCallback( vlc_object_t *p_this, char const *psz_cmd,
@@ -693,27 +784,27 @@ static int VisualizationCallback( vlc_object_t *p_this, char const *psz_cmd,
     aout_instance_t *p_aout = (aout_instance_t *)p_this;
     char *psz_mode = newval.psz_string;
     vlc_value_t val;
-    int i;
+    (void)psz_cmd; (void)oldval; (void)p_data;
 
     if( !psz_mode || !*psz_mode )
     {
-        ChangeFiltersString( p_aout, "audio-visual", "goom", VLC_FALSE );
-        ChangeFiltersString( p_aout, "audio-visual", "visual", VLC_FALSE );
-        ChangeFiltersString( p_aout, "audio-visual", "galaktos", VLC_FALSE );
+        ChangeFiltersString( p_aout, "audio-visual", "goom", false );
+        ChangeFiltersString( p_aout, "audio-visual", "visual", false );
+        ChangeFiltersString( p_aout, "audio-visual", "galaktos", false );
     }
     else
     {
         if( !strcmp( "goom", psz_mode ) )
         {
-            ChangeFiltersString( p_aout, "audio-visual", "visual", VLC_FALSE );
-            ChangeFiltersString( p_aout, "audio-visual", "goom", VLC_TRUE );
-            ChangeFiltersString( p_aout, "audio-visual", "galaktos", VLC_FALSE);
+            ChangeFiltersString( p_aout, "audio-visual", "visual", false );
+            ChangeFiltersString( p_aout, "audio-visual", "goom", true );
+            ChangeFiltersString( p_aout, "audio-visual", "galaktos", false);
         }
         else if( !strcmp( "galaktos", psz_mode ) )
         {
-            ChangeFiltersString( p_aout, "audio-visual", "visual", VLC_FALSE );
-            ChangeFiltersString( p_aout, "audio-visual", "goom", VLC_FALSE );
-            ChangeFiltersString( p_aout, "audio-visual", "galaktos", VLC_TRUE );
+            ChangeFiltersString( p_aout, "audio-visual", "visual", false );
+            ChangeFiltersString( p_aout, "audio-visual", "goom", false );
+            ChangeFiltersString( p_aout, "audio-visual", "galaktos", true );
         }
         else
         {
@@ -721,17 +812,14 @@ static int VisualizationCallback( vlc_object_t *p_this, char const *psz_cmd,
             var_Create( p_aout, "effect-list", VLC_VAR_STRING );
             var_Set( p_aout, "effect-list", val );
 
-            ChangeFiltersString( p_aout, "audio-visual", "goom", VLC_FALSE );
-            ChangeFiltersString( p_aout, "audio-visual", "visual", VLC_TRUE );
-            ChangeFiltersString( p_aout, "audio-visual", "galaktos", VLC_FALSE);
+            ChangeFiltersString( p_aout, "audio-visual", "goom", false );
+            ChangeFiltersString( p_aout, "audio-visual", "visual", true );
+            ChangeFiltersString( p_aout, "audio-visual", "galaktos", false);
         }
     }
 
     /* That sucks */
-    for( i = 0; i < p_aout->i_nb_inputs; i++ )
-    {
-        p_aout->pp_inputs[i]->b_restart = VLC_TRUE;
-    }
+    AoutInputsMarkToRestart( p_aout );
 
     return VLC_SUCCESS;
 }
@@ -742,13 +830,13 @@ static int EqualizerCallback( vlc_object_t *p_this, char const *psz_cmd,
     aout_instance_t *p_aout = (aout_instance_t *)p_this;
     char *psz_mode = newval.psz_string;
     vlc_value_t val;
-    int i;
     int i_ret;
+    (void)psz_cmd; (void)oldval; (void)p_data;
 
     if( !psz_mode || !*psz_mode )
     {
         i_ret = ChangeFiltersString( p_aout, "audio-filter", "equalizer",
-                                     VLC_FALSE );
+                                     false );
     }
     else
     {
@@ -756,18 +844,84 @@ static int EqualizerCallback( vlc_object_t *p_this, char const *psz_cmd,
         var_Create( p_aout, "equalizer-preset", VLC_VAR_STRING );
         var_Set( p_aout, "equalizer-preset", val );
         i_ret = ChangeFiltersString( p_aout, "audio-filter", "equalizer",
-                                     VLC_TRUE );
+                                     true );
 
     }
 
     /* That sucks */
     if( i_ret == 1 )
-    {
-        for( i = 0; i < p_aout->i_nb_inputs; i++ )
-        {
-            p_aout->pp_inputs[i]->b_restart = VLC_TRUE;
-        }
-    }
+        AoutInputsMarkToRestart( p_aout );
+    return VLC_SUCCESS;
+}
+
+static int ReplayGainCallback( vlc_object_t *p_this, char const *psz_cmd,
+                               vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    VLC_UNUSED(psz_cmd); VLC_UNUSED(oldval);
+    VLC_UNUSED(newval); VLC_UNUSED(p_data);
+    aout_instance_t *p_aout = (aout_instance_t *)p_this;
+    int i;
+
+    aout_lock_mixer( p_aout );
+    for( i = 0; i < p_aout->i_nb_inputs; i++ )
+        ReplayGainSelect( p_aout, p_aout->pp_inputs[i] );
+
+    /* Restart the mixer (a trivial mixer may be in use) */
+    aout_MixerMultiplierSet( p_aout, p_aout->mixer.f_multiplier );
+    aout_unlock_mixer( p_aout );
 
     return VLC_SUCCESS;
 }
+
+static void ReplayGainSelect( aout_instance_t *p_aout, aout_input_t *p_input )
+{
+    char *psz_replay_gain = var_GetNonEmptyString( p_aout,
+                                                   "audio-replay-gain-mode" );
+    int i_mode;
+    int i_use;
+    float f_gain;
+
+    p_input->f_multiplier = 1.0;
+
+    if( !psz_replay_gain )
+        return;
+
+    /* Find select mode */
+    if( !strcmp( psz_replay_gain, "track" ) )
+        i_mode = AUDIO_REPLAY_GAIN_TRACK;
+    else if( !strcmp( psz_replay_gain, "album" ) )
+        i_mode = AUDIO_REPLAY_GAIN_ALBUM;
+    else
+        i_mode = AUDIO_REPLAY_GAIN_MAX;
+
+    /* If the select mode is not available, prefer the other one */
+    i_use = i_mode;
+    if( i_use != AUDIO_REPLAY_GAIN_MAX && !p_input->replay_gain.pb_gain[i_use] )
+    {
+        for( i_use = 0; i_use < AUDIO_REPLAY_GAIN_MAX; i_use++ )
+        {
+            if( p_input->replay_gain.pb_gain[i_use] )
+                break;
+        }
+    }
+
+    /* */
+    if( i_use != AUDIO_REPLAY_GAIN_MAX )
+        f_gain = p_input->replay_gain.pf_gain[i_use] + var_GetFloat( p_aout, "audio-replay-gain-preamp" );
+    else if( i_mode != AUDIO_REPLAY_GAIN_MAX )
+        f_gain = var_GetFloat( p_aout, "audio-replay-gain-default" );
+    else
+        f_gain = 0.0;
+    p_input->f_multiplier = pow( 10.0, f_gain / 20.0 );
+
+    /* */
+    if( p_input->replay_gain.pb_peak[i_use] &&
+        var_GetBool( p_aout, "audio-replay-gain-peak-protection" ) &&
+        p_input->replay_gain.pf_peak[i_use] * p_input->f_multiplier > 1.0 )
+    {
+        p_input->f_multiplier = 1.0f / p_input->replay_gain.pf_peak[i_use];
+    }
+
+    free( psz_replay_gain );
+}
+
